'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var path = _interopDefault(require('path'));
var Metalsmith = _interopDefault(require('metalsmith'));
var match = _interopDefault(require('minimatch'));
var inquirer = _interopDefault(require('inquirer'));
var asyncEach = _interopDefault(require('async.each'));
var match$1 = _interopDefault(require('multimatch'));
var isBinaryPath = _interopDefault(require('is-binary-path'));
var cons = _interopDefault(require('consolidate'));

// forked from https://github.com/vuejs/vue-cli/blob/master/lib/eval.js

function evalualte(exp, data) {
  /* eslint-disable no-new-func */
  var fn = new Function('data', ("with (data) { return " + exp + " }"));
  try {
    return fn(data)
  } catch (err) {
    console.error(("Error when evaluating filter condition: " + exp));
  }
}

// forked from https://github.com/vuejs/vue-cli/blob/master/lib/filter.js
function filter(files, filters, data, done) {
  if (!filters) {
    return done()
  }
  var fileNames = Object.keys(files);
  Object.keys(filters).forEach(function (glob) {
    fileNames.forEach(function (file) {
      if (match(file, glob, {dot: true})) {
        var condition = filters[glob];
        if (!evalualte(condition, data)) {
          delete files[file];
        }
      }
    });
  });
  done();
}

function filterFiles(filters) {
  return function (files, metalsmith, done) {
    filter(files, filters, metalsmith.metadata(), done);
  }
}

function ask(data, prompts) {
  return function (files, metalsmith, done) {
    if (prompts) {
      inquirer.prompt(prompts)
        .then(function (answers) {
          metalsmith.metadata(Object.assign({}, data, answers));
          done();
        })
        .catch(done);
    } else if (data) {
      metalsmith.metadata(data);
      done();
    } else {
      done();
    }
  }
}

var getEngine = function (name) { return cons[name]; };

var template = function (ref) {
  if ( ref === void 0 ) ref = {};
  var skipInterpolation = ref.skipInterpolation;
  var engine = ref.engine;

  return function (files, metalsmith, done) {
    var keys = Object.keys(files);
    var matchedFile;
    if (skipInterpolation) {
      if (typeof skipInterpolation === 'function') {
        matchedFile = skipInterpolation;
      } else {
        var matches = match$1(keys, skipInterpolation);
        matchedFile = function (file) { return matches.indexOf(file) !== -1; };
      }
    }

    asyncEach(keys, run, done);

    function run(file, done) {
      var content = files[file].contents.toString();

      var shouldSkip = matchedFile && matchedFile(file, content);

      // we skip unmathed files (by multimatch or your own function)
      // and binary files
      if (shouldSkip || isBinaryPath(file)) {
        return done()
      }

      var renderer = getEngine(engine);

      renderer.render(content, metalsmith.metadata(), function (err, res) {
        if (err) {
          return done(err)
        }
        files[file].contents = new Buffer(res);
        done();
      });
    }
  }
};

function kopy(src, dest, ref) {
  if ( ref === void 0 ) ref = {};
  var cwd = ref.cwd; if ( cwd === void 0 ) cwd = process.cwd();
  var clean = ref.clean; if ( clean === void 0 ) clean = true;
  var data = ref.data;
  var prompts = ref.prompts;
  var skipInterpolation = ref.skipInterpolation;
  var engine = ref.engine; if ( engine === void 0 ) engine = 'handlebars';
  var filters = ref.filters;

  return new Promise(function (resolve, reject) {
    var source = path.resolve(cwd, src);
    Metalsmith(source) // eslint-disable-line new-cap
      .source('.')
      .ignore(function (file) { return /node_modules/.test(file); })
      .use(ask(data, prompts))
      .use(filterFiles(filters))
      .use(template({skipInterpolation: skipInterpolation, engine: engine}))
      .clean(clean)
      .destination(path.resolve(cwd, dest))
      .build(function (err, files) {
        if (err) { return reject(err) }
        resolve(Object.keys(files));
      });
  })
}

module.exports = kopy;
